@@ -23,7 +23,7 @@ function(speakerID, proposition, situation){
 			}
 			if(proposition$verb$type=='twoPlace'){
 				actor=ifelse(ACTOR(proposition$verb[,grep('^Ext\\d', names(proposition$verb))], proposition$verb[,grep('^Int\\d', names(proposition$verb))])==1, 'external', 'internal')
-				undergoer=ifelse(ACTOR(proposition$verb[,grep('^Ext\\d', names(proposition$verb))], proposition$verb[,grep('^Int\\d', names(proposition$verb))])==1, 'internal', 'external')
+				undergoer=ifelse(actor=='external', 'internal', 'external')
 				if(TRUE %in% c(sum(speaker$wordOrder[grep('UV', speaker$wordOrder$order, invert=TRUE), ]$success) < (sum(speaker$wordOrder$success)/log(sum(speaker$wordOrder$success))))){
 					if(grep(actor, names(proposition)) < grep('verb', names(proposition))){proposition=proposition[c(actor, undergoer, 'verb', 'target')]}
 					if(grep(actor, names(proposition)) > grep('verb', names(proposition))){proposition=proposition[c(undergoer, 'verb', actor, 'target')]}				
@@ -44,7 +44,9 @@ function(speakerID, proposition, situation){
 		if(proposition$verb$type=='onePlace'){
 			verbRole='external'
 			person=proposition$external$person
-			semRole='actor'
+			actorScore=VMATCH(proposition$verb[,grep('^Ext', names(proposition$verb))], rep(1, length(distinctions)))
+			undergoerScore=VMATCH(proposition$verb[,grep('^Ext', names(proposition$verb))], rep(0, length(distinctions)))
+			semRole=ifelse(undergoerScore<actorScore, 'actor', 'undergoer')
 			firstArgument=list(person=person, verbRole=verbRole, semRole=semRole, props=proposition$external[,grep('^D\\d', names(proposition$external))])		
 		}
 		if(proposition$verb$type=='twoPlace'){
@@ -68,8 +70,18 @@ function(speakerID, proposition, situation){
 			if(nrow(markers)==0){markers=speaker$nouns[speaker$nouns$person==firstArgument$person,]}
 			if(nrow(markers)!=0){
 				markers=markers[sample(nrow(markers)),]
-				if(firstArgument$verbRole=='external'){markers$match=VMATCH(proposition$verb[,grep('^Ext\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])}
-				if(firstArgument$verbRole=='internal'){markers$match=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])}
+				markers$collostruction=0
+				if(firstArgument$verbRole=='external'){
+					markers$match=VMATCH(proposition$verb[,grep('^Ext\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])
+					collostructions=speaker$collostructions$index[speaker$collostructions$index$N==proposition$external$ID,]
+					markers[markers$ID%in%collostructions$marker,]$collostruction=collostructions[na.omit(match(markers$ID, collostructions$marker)),]$frequency
+	
+				}
+				if(firstArgument$verbRole=='internal'){
+					markers$match=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])
+					collostructions=speaker$collostructions$index[speaker$collostructions$index$N==proposition$internal$ID,]
+					markers[markers$ID%in%collostructions$marker,]$collostruction=collostructions[na.omit(match(markers$ID, collostructions$marker)),]$frequency
+				}
 				markers=markers[order(CANDIDATESCORE(markers, type='verbMarker'), decreasing=TRUE),]
 				markerID=markers$ID[1]
 				markerTarget=proposition[[firstArgument$verbRole]]$ID
@@ -97,8 +109,17 @@ function(speakerID, proposition, situation){
 				markers=speaker$nouns[speaker$nouns$person==secondArgument$person & speaker$nouns$productionEffort<=suffixThreshold,]
 				if(nrow(markers)!=0){
 					markers=markers[sample(nrow(markers)),]
-					if(secondArgument$verbRole=='external'){markers$match=VMATCH(proposition$verb[,grep('^Ext\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])}
-					if(secondArgument$verbRole=='internal'){markers$match=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])}
+					markers$collostruction=0
+					if(secondArgument$verbRole=='external'){
+						markers$match=VMATCH(proposition$verb[,grep('^Ext\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])
+						collostructions=speaker$collostructions$index[speaker$collostructions$index$N==proposition$external$ID,]
+						markers[markers$ID%in%collostructions$marker,]$collostruction=collostructions[na.omit(match(markers$ID, collostructions$marker)),]$frequency
+					}
+					if(secondArgument$verbRole=='internal'){
+						markers$match=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])
+						collostructions=speaker$collostructions$index[speaker$collostructions$index$N==proposition$internal$ID,]
+						markers[markers$ID%in%collostructions$marker,]$collostruction=collostructions[na.omit(match(markers$ID, collostructions$marker)),]$frequency
+					}
 					markers=markers[order(CANDIDATESCORE(markers, type='verbMarker'), decreasing=TRUE),]
 					markerID=markers$ID[1]
 					markerTarget=proposition[[secondArgument$verbRole]]$ID
@@ -130,16 +151,20 @@ function(speakerID, proposition, situation){
 			topic=proposition[[firstArgument$verbRole]]$topic
 			pronouns=speaker$nouns[speaker$nouns$person==firstArgument$person & speaker$nouns$productionEffort>world$referenceThreshold,]
 			if(nrow(pronouns)>1){	
+				pronouns$collostruction=0
 				if(firstArgument$verbRole=='internal'){
 					pronouns$match=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], pronouns[,grep('^D\\d',names(pronouns))])
 					pronouns$typing=pronouns$match
+					collostructions=speaker$collostructions$OV[speaker$collostructions$OV$V==proposition$verb$ID,]
+					pronouns[pronouns$ID%in%collostructions$O,]$collostruction=collostructions[na.omit(match(pronouns$ID, collostructions$O)),]$frequency
 				}
 				if(firstArgument$verbRole=='external'){
 					pronouns$match=VMATCH(proposition$verb[,grep('^Ext\\d',names(proposition$verb))], pronouns[,grep('^D\\d',names(pronouns))])
 					pronouns$typing=pronouns$match
+					collostructions=speaker$collostructions$SV[speaker$collostructions$SV$V==proposition$verb$ID,]
+					pronouns[pronouns$ID%in%collostructions$S,]$collostruction=collostructions[na.omit(match(pronouns$ID, collostructions$S)),]$frequency
 				}
-				pronoun=pronouns[order(CANDIDATESCORE(pronouns, type='pronoun'), decreasing=TRUE)[1],]
-				pronoun$collostruction=pronoun$argument
+				pronoun=pronouns[order(CANDIDATESCORE(pronouns), decreasing=TRUE)[1],]
 				if(firstArgument$person!=3){
 					proposition[[firstArgument$verbRole]]=pronoun
 					proposition[[firstArgument$verbRole]]$topic=topic
@@ -155,31 +180,43 @@ function(speakerID, proposition, situation){
 			}	}	}
 			if(done==FALSE){
 				markers=speaker$nouns[sample(nrow(speaker$nouns)), ]
+				markers=markers[markers$person==3,]
+				markers$collostruction=0
 				if(firstArgument$verbRole=='internal'){
 					markers$match=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])
 					markers$distractorMatch=VMATCH(proposition$verb[,grep('^Ext\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])
+					collostructions=speaker$collostructions$OV[speaker$collostructions$OV$V==proposition$verb$ID,]
+					markers[markers$ID%in%collostructions$O,]$collostruction=collostructions[na.omit(match(markers$ID, collostructions$O)),]$frequency
 				}
 				if(firstArgument$verbRole=='external'){
 					markers$match=VMATCH(proposition$verb[,grep('^Ext\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])
-					markers$distractorMatch=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])		
+					if(proposition$verb$type=='onePlace'){					
+						markers$distractorMatch=.5
+					}
+					if(proposition$verb$type=='twoPlace'){					
+						markers$distractorMatch=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])		
+					}
+					collostructions=speaker$collostructions$SV[speaker$collostructions$SV$V==proposition$verb$ID,]
+					markers[markers$ID%in%collostructions$S,]$collostruction=collostructions[na.omit(match(markers$ID, collostructions$S)),]$frequency
 				}
-				markers=markers[markers$person==3,]
 				markers=markers[order(CANDIDATESCORE(markers, type='nounMarker'), decreasing=TRUE),]
 				markerID=0
 				for (i in 1:nrow(markers)){
 					if(markers[i,]$match > (markers[i,]$distractorMatch + distinctiveness)){
 					markerID=markers[i,]$ID
+					markerCollostruction=markers[i,]$collostruction
 						break()
 				}	}
-				if(markerID==0){markerID=markers[MAX(markers$match, forceChoice=TRUE),]$ID}
+				if(markerID==0){
+					markerID=markers[MAX(markers$match, forceChoice=TRUE),]$ID
+					markerCollostruction=markers[MAX(markers$match, forceChoice=TRUE),]$collostruction
+				}
 				proposition[[firstArgument$verbRole]]$markerID=markerID
 				proposition[[firstArgument$verbRole]]$marker=speaker$nouns[speaker$nouns$ID==markerID,]$form
 				if(frequency=='absolute'){proposition[[firstArgument$verbRole]]$markerFrequency=speaker$nouns[speaker$nouns$ID==markerID,]$frequency}
 				if(frequency=='relative'){proposition[[firstArgument$verbRole]]$markerFrequency=speaker$nouns[speaker$nouns$ID==markerID,]$nounMarker}
 				proposition[[firstArgument$verbRole]]$markerRecency=speaker$nouns[speaker$nouns$ID==markerID,]$recency
-				if(firstArgument$verbRole=='external'){markerCollostruction=speaker$collostructions$SV[intersect(grep(paste('^', markerID, '$', sep=''), speaker$collostructions$SV$S),grep(paste('^', proposition$verb$ID, '$', sep=''), speaker$collostructions$SV$V)),]$frequency}
-				if(firstArgument$verbRole=='internal'){markerCollostruction=speaker$collostructions$OV[intersect(grep(paste('^', markerID, '$', sep=''), speaker$collostructions$OV$O),grep(paste('^', proposition$verb$ID, '$', sep=''), speaker$collostructions$OV$V)),]$frequency}
-				if(length(markerCollostruction)!=0){proposition[[firstArgument$verbRole]]$markerCollostruction=markerCollostruction} else {proposition[[firstArgument$verbRole]]$markerCollostruction=0}			
+				proposition[[firstArgument$verbRole]]$markerCollostruction=markerCollostruction
 		}	}
 		if(proposition$verb$type=='twoPlace'){
 			data=speaker$usageHistory$flag$person[speaker$usageHistory$flag$person$role==secondArgument$semRole & speaker$usageHistory$flag$person$person==secondArgument$person,]
@@ -195,16 +232,20 @@ function(speakerID, proposition, situation){
 				topic=proposition[[secondArgument$verbRole]]$topic
 				pronouns=speaker$nouns[speaker$nouns$person==secondArgument$person & speaker$nouns$productionEffort>world$referenceThreshold,]
 				if(nrow(pronouns)>1){	
+					pronouns$collostruction=0
 					if(secondArgument$verbRole=='internal'){
 						pronouns$match=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], pronouns[,grep('^D\\d',names(pronouns))])
 						pronouns$typing=pronouns$match
+						collostructions=speaker$collostructions$OV[speaker$collostructions$OV$V==proposition$verb$ID,]
+						pronouns[pronouns$ID%in%collostructions$O,]$collostruction=collostructions[na.omit(match(pronouns$ID, collostructions$O)),]$frequency
 					}
 					if(secondArgument$verbRole=='external'){
 						pronouns$match=VMATCH(proposition$verb[,grep('^Ext\\d',names(proposition$verb))], pronouns[,grep('^D\\d',names(pronouns))])
 						pronouns$typing=pronouns$match
+						collostructions=speaker$collostructions$SV[speaker$collostructions$SV$V==proposition$verb$ID,]
+						pronouns[pronouns$ID%in%collostructions$S,]$collostruction=collostructions[na.omit(match(pronouns$ID, collostructions$S)),]$frequency
 					}
-					pronoun=pronouns[order(CANDIDATESCORE(pronouns, type='pronoun'), decreasing=TRUE)[1],]
-					pronoun$collostruction=pronoun$argument
+					pronoun=pronouns[order(CANDIDATESCORE(pronouns), decreasing=TRUE)[1],]
 					if(secondArgument$person!=3){
 						proposition[[secondArgument$verbRole]]=pronoun
 						proposition[[secondArgument$verbRole]]$topic=topic
@@ -221,30 +262,39 @@ function(speakerID, proposition, situation){
 				if(done==FALSE){
 					markers=speaker$nouns[speaker$nouns$person==3, ]
 					markers=markers[sample(nrow(markers)),]
+					markers$collostruction=0
 					if(secondArgument$verbRole=='internal'){
 						markers$match=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])
 						markers$distractorMatch=VMATCH(proposition$verb[,grep('^Ext\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])
+						collostructions=speaker$collostructions$OV[speaker$collostructions$OV$V==proposition$verb$ID,]
+						markers[markers$ID%in%collostructions$O,]$collostruction=collostructions[na.omit(match(markers$ID, collostructions$O)),]$frequency
 					}
 					if(secondArgument$verbRole=='external'){
 						markers$match=VMATCH(proposition$verb[,grep('^Ext\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])
 						markers$distractorMatch=VMATCH(proposition$verb[,grep('^Int\\d',names(proposition$verb))], markers[,grep('^D\\d',names(markers))])		
+						collostructions=speaker$collostructions$SV[speaker$collostructions$SV$V==proposition$verb$ID,]
+						markers[markers$ID%in%collostructions$S,]$collostruction=collostructions[na.omit(match(markers$ID, collostructions$S)),]$frequency
 					}
 					markers=markers[order(CANDIDATESCORE(markers, type='nounMarker'), decreasing=TRUE),]
 					markerID=0
 					for (i in 1:nrow(markers)){
 						if(markers[i,]$match > (markers[i,]$distractorMatch + distinctiveness)){
 						markerID=markers[i,]$ID
+						markerCollostruction=markers[i,]$collostruction
 							break()
 					}	}
-					if(markerID==0){markerID=markers[MAX(markers$match, forceChoice=TRUE),]$ID}
+					if(markerID==0){
+						markerID=markers[MAX(markers$match, forceChoice=TRUE),]$ID
+						markerCollostruction=markers[MAX(markers$match, forceChoice=TRUE),]$collostruction
+					}
 					proposition[[secondArgument$verbRole]]$markerID=markerID
 					proposition[[secondArgument$verbRole]]$marker=speaker$nouns[speaker$nouns$ID==markerID,]$form
 					if(frequency=='absolute'){proposition[[secondArgument$verbRole]]$markerFrequency=speaker$nouns[speaker$nouns$ID==markerID,]$frequency}
 					if(frequency=='relative'){proposition[[secondArgument$verbRole]]$markerFrequency=speaker$nouns[speaker$nouns$ID==markerID,]$nounMarker}
 					proposition[[secondArgument$verbRole]]$markerRecency=speaker$nouns[speaker$nouns$ID==markerID,]$recency
-					if(secondArgument$verbRole=='external'){markerCollostruction=speaker$collostructions$SV[intersect(grep(paste('^', markerID, '$', sep=''), speaker$collostructions$SV$S),grep(paste('^', proposition$verb$ID, '$', sep=''), speaker$collostructions$SV$V)),]$frequency}
-					if(secondArgument$verbRole=='internal'){markerCollostruction=speaker$collostructions$OV[intersect(grep(paste('^', markerID, '$', sep=''), speaker$collostructions$OV$O),grep(paste('^', proposition$verb$ID, '$', sep=''), speaker$collostructions$OV$V)),]$frequency}
-					if(length(markerCollostruction)!=0){proposition[[secondArgument$verbRole]]$markerCollostruction=markerCollostruction} else {proposition[[secondArgument$verbRole]]$markerCollostruction=0}			
+					proposition[[secondArgument$verbRole]]$markerCollostruction=markerCollostruction
 	}	}	}	}
 proposition	
-}
+}			
+
+
